#! /usr/bin/env python
# coding=utf-8
#================================================================
#   Copyright (C) 2019 * Ltd. All rights reserved.
#
#   Editor      : VIM
#   File name   : tools.py
#   Author      : YunYang1994
#   Created date: 2019-11-25 20:21:03
#   Description :
#
#================================================================


import cv2
import colorsys
import numpy as np
import imageio

def draw_2Dbox(image, bboxes, class_category):
    """
    image : numpy.ndarray
    bboxes: [cls_id, x_min, y_min, x_max, y_max] format coordinates.
    classes: ["person", "bus", ... ]
    """
    num_classes = len(class_category)
    fontScale = 0.5
    image_h, image_w, _ = image.shape
    hsv_tuples = [(1.0 * x / num_classes, 1., 1.) for x in range(num_classes)]

    colors = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    colors = list(map(lambda x: (int(x[0] * 255), int(x[1] * 255), int(x[2] * 255)), colors))

    for i, bbox in enumerate(bboxes):
        coor = np.array(bbox[1:5], dtype=np.int32)
        cls_id = int(bbox[0])
        bbox_color = colors[cls_id]
        c1, c2 = (coor[0], coor[1]), (coor[2], coor[3])
        cv2.rectangle(image, c1, c2, bbox_color, 2)

        bbox_mess = '%s' % class_category[cls_id]
        t_size = cv2.getTextSize(bbox_mess, 0, fontScale, thickness=1)[0]
        cv2.rectangle(image, c1, (c1[0] + t_size[0], c1[1] - t_size[1] - 3),
                      bbox_color, -1)  # filled
        cv2.putText(image, bbox_mess, (c1[0], c1[1]-2), cv2.FONT_HERSHEY_SIMPLEX,
                    fontScale, (0, 0, 0), 1, lineType=cv2.LINE_AA)
    return image

def create_gif(gif_name, frames, duration=0.5):
    '''
    creat GIF
    '''
    new_frames = []
    for image in frames:
        frames.append(imageio.imread(image))
    imageio.mimsave(gif_name, new_frames, 'GIF', duration = duration)
    return








from .version import __version__
from .tools import draw_2Dbox, create_gif
from .utils import read_text
from .augments import random_translate, random_crop, horizontal_flip, shift_gamma, shift_brightness, shift_color
from .augments import add_rain


def load_data():
    """
    Return:
        @image, a numpy array of shape (N, height, width, #channels)
        @boxes, a numpy array of shape (N, 5), representing N boxes
                "class_index, xmin, ymin, xmax, ymax"
        @classes, a list of class names
    """
    import os
    import cv2
    import numpy as np
    dirpath = os.path.dirname(os.path.abspath(__file__))
    image = cv2.imread(os.path.join(dirpath, "000998.png.py"))
    boxes = np.array([
        [  0.     , 582.99   , 180.48999, 615.14   , 211.09   ],
        [  3.     , 850.75   , 163.71   , 892.42   , 270.95   ],
        [  0.     , 696.89   , 182.62   , 826.96   , 273.94   ],
        [  0.     , 668.45996, 180.61   , 742.13995, 244.07   ],
        [  0.     , 642.69   , 175.2    , 679.51   , 210.86   ],
        [  0.     , 172.57999, 200.24   , 325.09998, 276.74   ],
        [  0.     , 282.46997, 189.66   , 387.62997, 249.08   ],
        [  0.     , 372.78   , 186.11   , 441.8    , 227.82   ],
        [  0.     , 631.13   , 176.96   , 651.28   , 196.17001]],)
    classes = ['Car', 'Truck', 'Van', 'Pedestrian']
    return image, boxes, classes


__all__ = [draw_2Dbox, read_text, load_data]



#! /usr/bin/env python
# coding=utf-8
#================================================================
#   Copyright (C) 2019 * Ltd. All rights reserved.
#
#   Editor      : VIM
#   File name   : augmets.py
#   Author      : YunYang1994
#   Created date: 2019-11-17 11:06:12
#   Description :
#
#================================================================

import cv2
import random
import numpy as np

def random_translate(image, bboxes, seed=1):
    random.seed(seed)
    h, w, _ = image.shape
    max_bbox = np.concatenate([np.min(bboxes[:, 1:3], axis=0), np.max(bboxes[:, 3:5], axis=0)], axis=-1)

    max_l_trans = max_bbox[0]
    max_u_trans = max_bbox[1]
    max_r_trans = w - max_bbox[2]
    max_d_trans = h - max_bbox[3]

    tx = random.uniform(-(max_l_trans - 1), (max_r_trans - 1))
    ty = random.uniform(-(max_u_trans - 1), (max_d_trans - 1))

    M = np.array([[1, 0, tx], [0, 1, ty]])
    image = cv2.warpAffine(image, M, (w, h))

    bboxes[:, [1, 3]] = bboxes[:, [1, 3]] + tx
    bboxes[:, [2, 4]] = bboxes[:, [2, 4]] + ty
    return image, bboxes

def random_crop(image, bboxes, seed=1):
    random.seed(seed)
    h, w, _ = image.shape
    max_bbox = np.concatenate([np.min(bboxes[:, 1:3], axis=0), np.max(bboxes[:, 3:5], axis=0)], axis=-1)

    max_l_trans = max_bbox[0]
    max_u_trans = max_bbox[1]
    max_r_trans = w - max_bbox[2]
    max_d_trans = h - max_bbox[3]

    crop_xmin = max(0, int(max_bbox[0] - random.uniform(0, max_l_trans)))
    crop_ymin = max(0, int(max_bbox[1] - random.uniform(0, max_u_trans)))
    crop_xmax = max(w, int(max_bbox[2] + random.uniform(0, max_r_trans)))
    crop_ymax = max(h, int(max_bbox[3] + random.uniform(0, max_d_trans)))

    image = image[crop_ymin : crop_ymax, crop_xmin : crop_xmax]

    bboxes[:, [1, 3]] = bboxes[:, [1, 3]] - crop_xmin
    bboxes[:, [2, 4]] = bboxes[:, [2, 4]] - crop_ymin

    return image, bboxes


def horizontal_flip(image, bboxes, seed=1):
    random.seed(seed)
    _, w, _ = image.shape
    flip_image = image[:, ::-1, :].copy()
    bboxes[:, [1,3]] = w - bboxes[:, [3,1]]

    return flip_image, bboxes

def shift_gamma(image, seed=1):
    random.seed(seed)
    # randomly shift gamma
    gamma = random.uniform(0.8, 1.2)
    print(gamma)
    image = image.copy() ** gamma
    image = np.clip(image, 0, 255)
    return image.astype(np.uint8)

def shift_brightness(image, seed=None):
    random.seed(seed)
    # randomly shift brightness
    brightness = random.uniform(0.5, 2.0)
    image = image.copy() * brightness
    image = np.clip(image, 0, 255)
    return image.astype(np.uint8)

def shift_color(image, seed=None):
    random.seed(seed)
    # randomly shift color
    colors = [random.uniform(0.8, 1.2) for _ in range(3)]
    white = np.ones([image.shape[0], image.shape[1]])
    color_image = np.stack([white * colors[i] for i in range(3)], axis=2)
    image = image.copy().astype(np.float32) * color_image.astype(np.float32)
    image = np.clip(image, 0, 255)
    return image.astype(np.uint8)

def get_noise(image, value, seed=None):
    noise = np.random.uniform(0,256,image.shape[0:2])
    noise[np.where(noise<(256-value))]=0

    k = np.array([ [0, 0.1, 0], [0.1,  8, 0.1], [0, 0.1, 0] ])

    noise = cv2.filter2D(noise,-1,k)
    return noise

def rain_blur(noise, length=10, angle=0, w=1):
    trans = cv2.getRotationMatrix2D((length/2, length/2), angle-45, 1-length/100.0)
    dig = np.diag(np.ones(length))
    k = cv2.warpAffine(dig, trans, (length, length))
    k = cv2.GaussianBlur(k,(w,w),0)
    blurred = cv2.filter2D(noise, -1, k)
    cv2.normalize(blurred, blurred, 0, 255, cv2.NORM_MINMAX)
    blurred = np.array(blurred, dtype=np.uint8)
    return blurred

def add_rain(img, beta = 0.8):
    noise = get_noise(img, value=1)
    rain = rain_blur(noise,length=20,angle=-30,w=3)
    rain = np.expand_dims(rain,2)
    rain_effect = np.concatenate((img,rain),axis=2)  #add alpha channel

    rain_result = img.copy()
    rain = np.array(rain,dtype=np.float32)
    rain_result[:,:,0] = rain_result[:,:,0] * (255-rain[:,:,0])/255 + beta*rain[:,:,0]
    rain_result[:,:,1] = rain_result[:,:,1] * (255-rain[:,:,0])/255 + beta*rain[:,:,0]
    rain_result[:,:,2] = rain_result[:,:,2] * (255-rain[:,:,0])/255 + beta*rain[:,:,0]
    return rain_result



#! /usr/bin/env python
# coding=utf-8
#================================================================
#   Copyright (C) 2019 * Ltd. All rights reserved.
#
#   Editor      : VIM
#   File name   : utils.py
#   Author      : YunYang1994
#   Created date: 2019-11-14 16:17:30
#   Description :
#
#================================================================

import numpy as np

def read_text(path):
    bboxes = []
    lines = open(path).readlines()
    for line in lines:
        label = line.split()
        bboxes.append(label)
    return np.array(bboxes, dtype=np.float32)





from __future__ import print_function
from setuptools import setup
exec(open('yyimg/version.py').read())
print("Version NO:", __version__)

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name = 'yyimg',
    packages = ['yyimg'],
    version = __version__,
    description = 'image tools for deep learning',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author = 'YunYang1994',
    author_email = 'dreameryangyun@sjtu.edu.cn',
    url = 'https://github.com/YunYang1994/yyimg.git',
    keywords = ['deep-learning', 'image'], # arbitrary keywords
    classifiers = [],
)

